/*
 * Copyright 2018 Chris Cromer
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * The main Sernatur namespace
 */
namespace Sernatur {
	using Postgres;

	/**
	 * The MainWindow class
	 */
	[GtkTemplate (ui = "/cl/cromer/ubb/sernatur/sernatur.window.ui")]
	public class MainWindow : Gtk.ApplicationWindow {

		/**
		 * This is a callback for when the window is maximized or unmaximized
		 * @param widget The widget that called this GtkCallback
		 * @param state The event window state_handler
		 * @return Returns true if the event is handled or false if it isn't
		 */
		[GtkCallback]
		private bool maximize_window(Gtk.Widget widget, Gdk.EventWindowState state) {
			Settings settings = new Settings ("cl.cromer.ubb.sernatur.window");
			if (state.changed_mask == Gdk.WindowState.MAXIMIZED && (state.new_window_state & Gdk.WindowState.MAXIMIZED) != 0) {
				// Maximized
				settings.set_boolean ("maximized", true);
			}
			else if (state.changed_mask == Gdk.WindowState.MAXIMIZED && (state.new_window_state & Gdk.WindowState.MAXIMIZED) == 0) {
				// Unmaximized
				settings.set_boolean ("maximized", false);
			}
			return false;
		}

		/**
		 * This is a callback for when the window's size is changed
		 * @param widget The widget that called this GtkCallback
		 * @param event The event that happened
		 * @return Returns true if the event is handled or false if it isn't
		 */
		[GtkCallback]
		private bool resize_window(Gtk.Widget widget, Gdk.Event event) {
			Settings settings = new Settings ("cl.cromer.ubb.sernatur.window");
			if (event.get_event_type () == Gdk.EventType.CONFIGURE && !settings.get_boolean ("maximized")) {
				if (settings.get_int ("width") != event.get_window ().get_width ()) {
					settings.set_int ("width", event.get_window ().get_width ());
				}
				if (settings.get_int ("height") != event.get_window ().get_height ()) {
					settings.set_int ("height", event.get_window ().get_height ());
				}
			}
			return false;
		}

		/**
		* Initialize the main window class
		* @param application The application used to make the GLib object
		*/
		public MainWindow (Gtk.Application application) {
			GLib.Object (application: application);
			Database conn;

			Settings settings = new Settings ("cl.cromer.ubb.sernatur.db");
			string host = settings.get_string ("host");
			string port = settings.get_string ("port");
			string options = settings.get_string ("options");
			string tty = settings.get_string ("tty");
			string database = settings.get_string ("database");
			string username = settings.get_string ("username");
			string password = settings.get_string ("password");

			conn = set_db_login (host, port, options, tty, database, username, password);
			if (conn.get_status () != ConnectionStatus.OK) {
				stderr.printf ("%s\n", conn.get_error_message ());
				Gtk.MessageDialog msg = new Gtk.MessageDialog (this,
					Gtk.DialogFlags.MODAL,
					Gtk.MessageType.ERROR,
					Gtk.ButtonsType.CLOSE,
					conn.get_error_message ());
				msg.response.connect ((response_id) => {
					switch (response_id) {
						case Gtk.ResponseType.CLOSE:
							application.quit ();
							break;
						case Gtk.ResponseType.DELETE_EVENT:
							application.quit ();
							break;
					}
					msg.destroy();
				});
				msg.set_title (dgettext (null, "Error"));
				msg.show ();
				return;
			}

			/*Result res = conn.exec ("CREATE DATABASE bdd;");
			if (res.get_status () != ExecStatus.COMMAND_OK) {
				stderr.printf ("%s\n", conn.get_error_message ());
				application.quit ();
				return;
			}

			string password = encrypt_password ("bdd", "bdd");

			res = conn.exec ("CREATE USER bdd WITH ENCRYPTED PASSWORD '" + password + "';");
			if (res.get_status () != ExecStatus.COMMAND_OK) {
				stderr.printf ("%s\n", conn.get_error_message ());
				application.quit ();
				return;
			}

			res = conn.exec ("GRANT ALL PRIVILEGES ON DATABASE bdd TO bdd;");
			if (res.get_status () != ExecStatus.COMMAND_OK) {
				stderr.printf ("%s\n", conn.get_error_message ());
				application.quit ();
				return;
			}*/

			GLib.print (dgettext (null, "Server version:") + " %d\n", conn.get_server_version ());
		}
	}
}
